﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using I8123W_Net;

namespace PDO_Parameter
{
    public partial class Form1 : Form
    {
        byte SLOT;
        byte BAUD;
        byte NODE;
        
        byte BlockMode = 1;

        byte PDOTYPE;
        ushort PDOCOBID;

        ushort DelayTime;
        ushort ret;
        string errstr;

        public Form1()
        {
            InitializeComponent();
        }

        private void GetPDOID()
        {
            byte PDO_Cnt;
            ushort [] Id_List = new ushort[10]; 
            ushort i;

            Combo_PDOID.Items.Clear();

            if (PDOTYPE != 0)
            {
                ret = I8123W.I8123_GetTxPDOID(SLOT, NODE, out PDO_Cnt, Id_List, BlockMode);
            }
            else
            {
                ret = I8123W.I8123_GetRxPDOID(SLOT, NODE, out PDO_Cnt, Id_List, BlockMode);
            }

            if (ret == 0)
            {
                for (i=0;i<PDO_Cnt;i++)
                {
                    Combo_PDOID.Items.Add(Id_List[i].ToString("X3"));
                }
            }

            if (PDO_Cnt != 0) Combo_PDOID.SelectedIndex = 0;

            GetPDOMapData();
        }

        private void GetPDOMapData()
        {
            ushort PDO_Index, PDONo, i;
            byte PDO_Type, Transmit_Type, Entry_Cnt;
            ushort Event_Timer;
            uint [] Map_Data = new uint[10];

            List_MapData.Items.Clear();

            PDOCOBID = 0;

            if (Combo_PDOID.Items.Count == 0)
            {
                return;
            }

            PDO_Index = (ushort)(Combo_PDOID.SelectedIndex);

            // Convert text to PDO ID number
            PDOCOBID = Convert.ToUInt16(Combo_PDOID.Text, 16);

            // Get Mapping Data
            ret = I8123W.I8123_GetPDOMapInfo(SLOT, PDOCOBID, out PDONo, out PDO_Type, out Transmit_Type, out Event_Timer, out Entry_Cnt, Map_Data, BlockMode);
            if (ret == 0)
            {
                List_MapData.Items.Add("Transmission Type: " + Transmit_Type.ToString("D"));

                List_MapData.Items.Add("Event Timer: " + Event_Timer.ToString("D"));

                List_MapData.Items.Add("Mapping Data: ");

                if (Entry_Cnt == 0)
                {
                    List_MapData.Items.Add("None");
                }
                else
                {
                    for (i = 0; i < Entry_Cnt; i++)
                    {
                        List_MapData.Items.Add("Entry " + (i + 1).ToString("D") + ": " + Map_Data[i].ToString("X8"));
                    }
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SLOT = 0;
            BAUD = 0;
            NODE = 1;

            PDOCOBID = 0;
            PDOTYPE = 0;

            Combo_Slot.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = false;

            Combo_Slot.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;

            Btn_AddNode.Enabled = false;
            Btn_AddNode.Visible = true;
            Btn_Remove.Enabled = false;

            Pal_PDOInstall.Enabled = false;
            Pal_PDOParameter.Enabled = false;
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            I8123W.I8123_ShutdownMaster(SLOT);
        }

        private void Combo_Slot_SelectedIndexChanged(object sender, EventArgs e)
        {
            SLOT = (byte)(Combo_Slot.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);

            switch (BAUD)
            {
                case 0:
                    DelayTime = 15;
                    break;
                case 1:
                    DelayTime = 7;
                    break;
                case 2:
                    DelayTime = 3;
                    break;
                case 3:
                case 4:
                    DelayTime = 1;
                    break;
                case 5:
                case 6:
                case 7:
                    DelayTime = 0;
                    break;
            }
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);
        }

        private void Combo_PDOType_SelectedIndexChanged(object sender, EventArgs e)
        {
            PDOTYPE = (byte)(Combo_PDOType.SelectedIndex);

            // Get total RxPDO COB-ID of the node
            GetPDOID();
        }

        private void Combo_PDOID_SelectedIndexChanged(object sender, EventArgs e)
        {
            GetPDOMapData();
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_InitMaster(SLOT, 0, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_ShutdownMaster(SLOT);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;

                Pal_PDOInstall.Enabled = false;
                Pal_PDOParameter.Enabled = false;
            }
        }

        private void Btn_AddNode_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_AddNode(SLOT, NODE, 1, DelayTime, 200, BlockMode);
            if (ret != 0)
            {
                errstr = "AddNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_Remove.Enabled = true;
                Combo_Node.Enabled = false;

                Combo_PDOType.SelectedIndex = 0;
                Combo_Install_Type.SelectedIndex = 0;

                Pal_PDOInstall.Enabled = true;
                Pal_PDOParameter.Enabled = true;

                GetPDOID();
            }
        }

        private void Btn_Remove_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_RemoveNode(SLOT, NODE, BlockMode);
            if (ret != 0)
            {
                errstr = "RemoveNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;
                Combo_Node.Enabled = true;

                Pal_PDOInstall.Enabled = false;
                Pal_PDOParameter.Enabled = false;
            }
        }

        private void Check_Is_Dec(object sender, KeyPressEventArgs e)
        {
            if (!(("1234567890".IndexOf(e.KeyChar) != -1) || ((int)e.KeyChar == 8))) e.Handled = true;
        }

        private void Check_Is_Hex(object sender, KeyPressEventArgs e)
        {
            if (!(("1234567890ABCDEFabcdef".IndexOf(e.KeyChar) != -1) || ((int)e.KeyChar == 8))) e.Handled = true;
        }

        private void Btn_InstallPDO_Click(object sender, EventArgs e)
        {
            byte RxTx;
            ushort CobId, PDO_No;

            if (Text_Install_ID.Text == "") Text_Install_ID.Text = "0";
            CobId = Convert.ToUInt16(Text_Install_ID.Text, 16);

            if (Text_Install_No.Text == "") Text_Install_No.Text = "0";
            PDO_No = Convert.ToUInt16(Text_Install_No.Text, 10);

            RxTx = (byte)(Combo_Install_Type.SelectedIndex);

            ret = I8123W.I8123_InstallPDO(SLOT, NODE, CobId, RxTx, PDO_No, BlockMode);
            if (ret != 0)
            {
                errstr = "InstallPDO Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                if (PDOTYPE == RxTx)
                {
                    GetPDOID();
                }
            }
        }

        private void Btn_RemovePDO_Click(object sender, EventArgs e)
        {
            byte Entry;

            if (Text_Entry.Text == "") Text_Entry.Text = "0";
            Entry = Convert.ToByte(Text_Entry.Text, 10);

            ret = I8123W.I8123_RemovePDO(SLOT, NODE, PDOCOBID, Entry, BlockMode);
            if (ret != 0)
            {
                errstr = "RemovePDO Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                if (Entry != 0)   // Entry != 0, Remove single entry
                {
                    GetPDOMapData();
                }
                else              // Entry = 0, Remove whole PDO
                {
                    GetPDOID();
                }
            }
        }

        private void Btn_DynamicPDO_Click(object sender, EventArgs e)
        {
            byte Entry;
            uint EntryData;

            if (Text_Entry.Text == "") Text_Entry.Text = "0";
            Entry = Convert.ToByte(Text_Entry.Text, 10);

            if (Text_EntryData.Text == "") Text_EntryData.Text = "0";
            EntryData = Convert.ToUInt32(Text_EntryData.Text, 16);

            ret = I8123W.I8123_DynamicPDO(SLOT, NODE, PDOCOBID, PDOTYPE, Entry, EntryData, BlockMode);
            if (ret != 0)
            {
                errstr = "DynamicPDO Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                GetPDOMapData();
            }
        }

        private void Btn_ChangePDOID_Click(object sender, EventArgs e)
        {
            ushort New_PDO;

            if (Text_NewID.Text == "") Text_NewID.Text = "0";
            New_PDO = Convert.ToUInt16(Text_NewID.Text, 16);

            ret = I8123W.I8123_ChangePDOID(SLOT, PDOCOBID, New_PDO, BlockMode);
            if (ret != 0)
            {
                errstr = "ChangePDOID Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                GetPDOID();
            }
        }

        private void Btn_SetTxType_Click(object sender, EventArgs e)
        {
            byte Tx_Type;

            if (Text_TransmitType.Text == "") Text_TransmitType.Text = "0";
            Tx_Type = Convert.ToByte(Text_TransmitType.Text, 10);

            ret = I8123W.I8123_PDOTxType(SLOT, PDOCOBID, Tx_Type, BlockMode);
            if (ret != 0)
            {
                errstr = "PDOTxType Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                GetPDOMapData();
            }
        }

        private void Btn_SetEntryNo_Click(object sender, EventArgs e)
        {
            byte Entry;

            if (Text_PDOEntryNo.Text == "") Text_PDOEntryNo.Text = "0";
            Entry = Convert.ToByte(Text_PDOEntryNo.Text, 10);

            ret = I8123W.I8123_PDOUseEntry(SLOT, PDOCOBID, Entry, BlockMode);
            if (ret != 0)
            {
                errstr = "PDOUseEntr Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                GetPDOMapData();
            }
        }

        private void Btn_SetEventTime_Click(object sender, EventArgs e)
        {
            ushort Timer;

            if (Text_EventTimer.Text == "") Text_EventTimer.Text = "0";
            Timer = Convert.ToUInt16(Text_EventTimer.Text,10);

            ret = I8123W.I8123_PDOEventTimer(SLOT, PDOCOBID, Timer, BlockMode);
            if (ret != 0)
            {
                errstr = "PDOEventTimer Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                GetPDOMapData();
            }
        }
    }
}