﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using I8123W_Net;

namespace PDO_Protocol
{
    public partial class Form1 : Form
    {
        byte SLOT;
        byte BAUD;
        byte NODE;

        byte BlockMode = 1;

        // Set PDO Data Array
        TextBox[] PDO_Data;

        ushort DelayTime;
        ushort ret;
        string errstr;

        public Form1()
        {
            InitializeComponent();

            PDO_Data = new TextBox[8] { Text_D0, Text_D1, Text_D2, Text_D3, Text_D4, Text_D5, Text_D6, Text_D7 };
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SLOT = 0;
            BAUD = 0;
            NODE = 1;

            Combo_Slot.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = false;

            Combo_Slot.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;

            Btn_AddNode.Enabled = false;
            Btn_AddNode.Visible = true;
            Btn_Remove.Enabled = false;

            Pal_PDO.Enabled = false;
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            I8123W.I8123_ShutdownMaster(SLOT);
        }

        private void Combo_Slot_SelectedIndexChanged(object sender, EventArgs e)
        {
            SLOT = (byte)(Combo_Slot.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);

            switch (BAUD)
            {
                case 0:
                    DelayTime = 15;
                    break;
                case 1:
                    DelayTime = 7;
                    break;
                case 2:
                    DelayTime = 3;
                    break;
                case 3:
                case 4:
                    DelayTime = 1;
                    break;
                case 5:
                case 6:
                case 7:
                    DelayTime = 0;
                    break;
            }
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_InitMaster(SLOT, 0, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_ShutdownMaster(SLOT);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;

                Pal_PDO.Enabled = false;

                Timer1.Enabled = false;
            }
        }

        private void Btn_AddNode_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_AddNode(SLOT, NODE, 1, DelayTime, 200, BlockMode);
            if (ret != 0)
            {
                errstr = "AddNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_Remove.Enabled = true;
                Combo_Node.Enabled = false;

                Combo_PDOLen.SelectedIndex = 0;

                Pal_PDO.Enabled = true;

                Timer1.Enabled = true;
            }
        }

        private void Btn_Remove_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_RemoveNode(SLOT, NODE, BlockMode);
            if (ret != 0)
            {
                errstr = "RemoveNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;
                Combo_Node.Enabled = true;

                Pal_PDO.Enabled = false;

                Timer1.Enabled = false;
            }
        }

        private void Btn_Clear_Click(object sender, EventArgs e)
        {
            List_PDOData.Items.Clear();
        }

        private void Check_Is_Hex(object sender, KeyPressEventArgs e)
        {
            if (!(("1234567890ABCDEFabcdef".IndexOf(e.KeyChar) != -1) || ((int)e.KeyChar == 8))) e.Handled = true;
        }

        private void Btn_PDOWrite_Click(object sender, EventArgs e)
        {
            ushort i, CobId;
            byte Len;
            byte[] TData = new byte[8];
            string DataStr;

            CobId = Convert.ToUInt16(Text_COBID.Text, 16);
            Len = (byte)(Combo_PDOLen.SelectedIndex + 1);

            for (i = 0; i < 8; i++)
            {
                TData[i] = Convert.ToByte(PDO_Data[i].Text, 16);
            }

            ret = I8123W.I8123_PDOWrite(SLOT, CobId, 0, Len, TData, BlockMode);
            if (ret != 0)
            {
                errstr = "PDO Write Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                DataStr = Text_COBID.Text + ": ";

                for (i = 0; i < Len; i++)
                {
                    DataStr = DataStr + TData[i].ToString("X2") + " ";
                }

                DataStr = DataStr + ": Write OK";

                List_PDOData.Items.Add(DataStr);
            }
        }

        private void Btn_PDORemote_Click(object sender, EventArgs e)
        {
            ushort i, CobId;
            byte RLen;
            byte [] RData = new byte[8];
            string DataStr;

            CobId = Convert.ToUInt16(Text_COBID.Text, 16);

            ret = I8123W.I8123_PDORemote(SLOT, CobId, out RLen, RData, BlockMode);
            if (ret != 0)
            {
                errstr = "PDO Remote Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                DataStr = Text_COBID.Text + ": ";

                for (i = 0; i < RLen; i++)
                {
                    DataStr = DataStr + RData[i].ToString("X2") + " ";
                }

                DataStr = DataStr + ": Remote OK";

                List_PDOData.Items.Add(DataStr);
            }
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            ushort Ret, i;
            byte IsNew, Dlen;
            byte [] Data = new byte[8];
            string DataStr;

            // Check PDO data of default TxPDO  
            Ret = I8123W.I8123_GetPDOLastData(SLOT, (ushort)(0x180 + NODE), out IsNew, out Dlen, Data, BlockMode);
            if ((Ret == 0) && (IsNew == 1))
            {
                DataStr = (0x180 + NODE).ToString("X") + ": ";

                for (i = 0; i < Dlen; i++)
                {
                    DataStr = DataStr + Data[i].ToString("X2") + " ";
                }

                DataStr = DataStr + ": Get TxPDO";

                List_PDOData.Items.Add(DataStr);
            }

            Ret = I8123W.I8123_GetPDOLastData(SLOT, (ushort)(0x280 + NODE), out IsNew, out Dlen, Data, BlockMode);
            if ((Ret == 0) && (IsNew == 1))
            {
                DataStr = (0x280 + NODE).ToString("X") + ": ";

                for (i = 0; i < Dlen; i++)
                {
                    DataStr = DataStr + Data[i].ToString("X2") + " ";
                }

                DataStr = DataStr + ": Get TxPDO";

                List_PDOData.Items.Add(DataStr);
            }

            Ret = I8123W.I8123_GetPDOLastData(SLOT, (ushort)(0x380 + NODE), out IsNew, out Dlen, Data, BlockMode);
            if ((Ret == 0) && (IsNew == 1))
            {
                DataStr = (0x380 + NODE).ToString("X") + ": ";

                for (i = 0; i < Dlen; i++)
                {
                    DataStr = DataStr + Data[i].ToString("X2") + " ";
                }

                DataStr = DataStr + ": Get TxPDO";

                List_PDOData.Items.Add(DataStr);
            }

            Ret = I8123W.I8123_GetPDOLastData(SLOT, (ushort)(0x480 + NODE), out IsNew, out Dlen, Data, BlockMode);
            if ((Ret == 0) && (IsNew == 1))
            {
                DataStr = (0x480 + NODE).ToString("X") + ": ";

                for (i = 0; i < Dlen; i++)
                {
                    DataStr = DataStr + Data[i].ToString("X2") + " ";
                }

                DataStr = DataStr + ": Get TxPDO";

                List_PDOData.Items.Add(DataStr);
            }
        }
    }
}