﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WinPacNet;
using I8123W_Net;

namespace SDO_PDO_ISR
{
    public partial class Form1 : Form
    {
        const int DI_MODULE_SLOT = 2;
        const int DI_TOTAL_CH = 16;

        const int DO_MODULE_SLOT = 1;
        const int DO_TOTAL_CH = 16;

        byte SLOT;
        byte BAUD;
        byte NODE;

        byte BlockMode = 1;

        string ISR_Str = "";
        IntPtr HPORT;
        
        ushort ret;
        string errstr;

        I8123W.ReadSDOISR RSDOISR;
        I8123W.WriteSDOISR WSDOISR;
        I8123W.RxPDOISR RXPDOISR;
        I8123W.RemotePDOISR REMOTEPDOISR;

        public Form1()
        {
            InitializeComponent();
        }

        private void UserRSDOISR()
        {
            ushort Ret, Index;
            byte SlotNo, SubIndex, RLen;
            byte[] RData = new byte[4];
            uint Abort = 0;

            Ret = I8123W.I8123_GetMasterReadSDOEvent(out SlotNo, out Index, out SubIndex);
            if (Ret == 0)
            {
                switch (Index)
                {
                    case 0x6000:	// Read DI
                        {
                            uint DI_Value = 0;

                            switch (SubIndex)
                            {
                                case 0x00:
                                    {
                                        // DI entry number
                                        RLen = 1;
                                        RData[0] = 0x02;	// ch0~ch15, total 2-bytes (16-bits) DI data.
                                    }
                                    break;
                                case 0x01:
                                    {
                                        RLen = 1;

                                        // Read DI value of ch0 ~ ch7
                                        WinPAC.pac_ReadDI(HPORT, DI_MODULE_SLOT, DI_TOTAL_CH, ref DI_Value);
                                        RData[0] = (byte)(DI_Value & 0x000000FF);
                                    }
                                    break;
                                case 0x02:
                                    {
                                        RLen = 1;

                                        // Read DI value of ch8 ~ ch15
                                        WinPAC.pac_ReadDI(HPORT, DI_MODULE_SLOT, DI_TOTAL_CH, ref DI_Value);
                                        RData[0] = (byte)((DI_Value >> 8) & 0x000000FF);
                                    }
                                    break;
                                default:
                                    {
                                        Abort = 0x06090011;

                                        // Abort: Sub-index does not exist.
                                        RLen = 4;
                                        RData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                            }
                        }
                        break;
                    case 0x6200:	// Read DO
                        {
                            uint DO_Value = 0;

                            switch (SubIndex)
                            {
                                case 0x00:
                                    {
                                        // DO entry number
                                        RLen = 1;
                                        RData[0] = 0x02;   // ch0~ch15, total 2-bytes (16-bits) DO data.
                                    }
                                    break;
                                case 0x01:
                                    {
                                        RLen = 1;

                                        // Read DO value of ch0 ~ ch7
                                        WinPAC.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, ref DO_Value);
                                        RData[0] = (byte)(DO_Value & 0x000000FF);
                                    }
                                    break;
                                case 0x02:
                                    {
                                        RLen = 1;

                                        // Read DO value of ch8 ~ ch15
                                        WinPAC.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, ref DO_Value);
                                        RData[0] = (byte)((DO_Value >> 8) & 0x000000FF);
                                    }
                                    break;
                                default:
                                    {
                                        Abort = 0x06090011;

                                        // Abort: Sub-index does not exist.
                                        RLen = 4;
                                        RData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                            }
                        }
                        break;
                    default:
                        {
                            Abort = 0x06020000;

                            // Abort: Object does not exist.
                            RLen = 4;
                            RData = BitConverter.GetBytes(Abort);
                        }
                        break;
                }

                {
                    ushort i;
                    string TempStr;

                    if (Abort == 0)
                    {
                        I8123W.I8123_ResponseMasterSDO(SlotNo, 0, Index, SubIndex, RLen, RData);
                        TempStr = "SDO Read";
                    }
                    else
                    {
                        I8123W.I8123_SDOAbortTransmit(SlotNo, NODE, Index, SubIndex, Abort, BlockMode);
                        TempStr = "SDO Abort";
                    }


                    TempStr = TempStr + " " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":";
                    for (i = 0; i < RLen; i++)
                    {
                        TempStr = TempStr + " " + RData[i].ToString("X2");
                    }

                    ISR_Str = TempStr;
                }
            }

            return;
        }
        
        private void UserWSDOISR()
        {
            ushort Ret, Index;
            byte SlotNo, SubIndex, WLen;
            byte[] WData = new byte[4] { 0, 0, 0, 0 };
            uint Abort = 0;

            Ret = I8123W.I8123_GetMasterWriteSDOEvent(out SlotNo, out Index, out SubIndex, out WLen, WData);
            if (Ret == 0)
            {
                switch (Index)
                {
                    case 0x6200:	// Write DO
                        {
                            uint DO_Value = 0;

                            if (WLen != 1)
                            {
                                Abort = 0x06070010;

                                // Abort: Data type does not match, length of service parameter does not match
                                WLen = 4;
                                WData = BitConverter.GetBytes(Abort);
                            }

                            switch (SubIndex)
                            {
                                case 0x00:
                                    {
                                        Abort = 0x06010002;

                                        // Abort: Attempt to write a read only object
                                        WLen = 4;
                                        WData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                                case 0x01:
                                    {
                                        uint temp_value;
                                        // Get original DO value
                                        WinPAC.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, ref DO_Value);

                                        // Change DO value of ch0 ~ ch7
                                        temp_value = WData[0];
                                        DO_Value = (DO_Value & 0xFFFFFF00) | (temp_value & 0x000000FF);
                                        WinPAC.pac_WriteDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value);

                                        // Write OK
                                    }
                                    break;
                                case 0x02:
                                    {
                                        uint temp_value;
                                        // Get original DO value
                                        WinPAC.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, ref DO_Value);

                                        // Change DO value of ch8 ~ ch15
                                        temp_value = WData[0];
                                        DO_Value = (DO_Value & 0xFFFF00FF) | ((temp_value << 8) & 0x0000FF00);
                                        WinPAC.pac_WriteDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value);

                                        // Write OK
                                    }
                                    break;
                                default:
                                    {
                                        Abort = 0x06090011;

                                        // Abort: Sub-index does not exist.
                                        WLen = 4;
                                        WData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                            }
                        }
                        break;
                    default:
                        {
                            Abort = 0x06020000;

                            // Abort: Object does not exist.
                            WLen = 4;
                            WData = BitConverter.GetBytes(Abort);
                        }
                        break;
                }

                {
                    ushort i;
                    string TempStr;

                    if (Abort == 0)
                    {
                        I8123W.I8123_ResponseMasterSDO(SlotNo, 1, Index, SubIndex, 0, null);
                        TempStr = "SDO Write " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ": OK";
                    }
                    else
                    {
                        I8123W.I8123_SDOAbortTransmit(SlotNo, NODE, Index, SubIndex, Abort, BlockMode);
                        TempStr = "SDO Abort " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":";

                        for (i = 0; i < WLen; i++)
                        {
                            TempStr = TempStr + " " + WData[i].ToString("X2");
                        }
                    }

                    ISR_Str = TempStr;
                }
            }

            return;
        }

        private void UserRXPDOISR()
        {
            ushort Ret;
            byte SlotNo, WLen;
            ushort CobId, i;
            byte[] WData = new byte[8] { 0, 0, 0, 0, 0, 0, 0, 0 };
            string TempStr;

            Ret = I8123W.I8123_GetMasterRxPDOEvent(out SlotNo, out CobId, out WLen, WData);
            if (Ret == 0)
            {
                if (CobId == 0x200 + NODE)
                {
                    if (WLen >= 2)
                    {
                        uint DO_Value;

                        // Output DO value of ch0 ~ ch15
                        DO_Value = BitConverter.ToUInt32(WData, 0);
                        WinPAC.pac_WriteDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value);
                        
                        TempStr = "PDO " + CobId.ToString("X3") + " Output:";
                        for (i = 0; i < WLen; i++)
                        {
                            TempStr = TempStr + " " + WData[i].ToString("X2");
                        }

                        ISR_Str = TempStr;
                    }
                }
            }
        }

        private void  UserREMOTEPDOISR()
        {
            ushort Ret;
            byte SlotNo, RLen;
            ushort CobId, i;
            byte[] RData = new byte[8] { 0, 0, 0, 0, 0, 0, 0, 0 };
            string TempStr;

            Ret = I8123W.I8123_GetMasterRemotePDOEvent(out SlotNo, out CobId);
            if (Ret == 0)
            {
                if (CobId == 0x180 + NODE)
                {
                    uint DI_Value = 0;

                    // Read DI value of ch0 ~ ch15
                    WinPAC.pac_ReadDI(HPORT, DI_MODULE_SLOT, DI_TOTAL_CH, ref DI_Value);

                    RLen = 2;
                    RData = BitConverter.GetBytes(DI_Value);

                    TempStr = "PDO " + CobId.ToString("X3") + " Remote:";
                    for (i = 0; i < RLen; i++)
                    {
                        TempStr = TempStr + " " + RData[i].ToString("X2");
                    }

                    ISR_Str = TempStr;
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SLOT = 0;
            BAUD = 0;
            NODE = 1;

            Combo_Slot.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = true;

            Combo_Slot.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;

            Btn_Clear.Enabled = false;

            HPORT = WinPAC.uart_Open("COM0:,115200,N,8,1");
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            I8123W.I8123_ShutdownMaster(SLOT);
        }

        private void Combo_Slot_SelectedIndexChanged(object sender, EventArgs e)
        {
            SLOT = (byte)(Combo_Slot.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            uint [] Entry_Data = new uint[8];

            ret = I8123W.I8123_InitMaster(SLOT, NODE, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_Clear.Enabled = true;

                // Install SDO read ISR
                RSDOISR = new I8123W.ReadSDOISR(UserRSDOISR);
                ret = I8123W.I8123_InstallReadSDOISR(SLOT, RSDOISR);
                if (ret != 0)
                {
                    errstr = "Install Read SDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }
                
                // Initial SDO write ISR
                WSDOISR = new I8123W.WriteSDOISR(UserWSDOISR);
                ret = I8123W.I8123_InstallWriteSDOISR(SLOT, WSDOISR);
                if (ret != 0)
                {
                    errstr = "Install Write SDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }
                
                // Install PDO output ISR
                RXPDOISR = new I8123W.RxPDOISR(UserRXPDOISR);
                ret = I8123W.I8123_InstallRxPDOISR(SLOT, RXPDOISR);
                if (ret != 0)
                {
                    errstr = "Install output PDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }

                // The "EntryUse" parameter and "EntryData" parameter are useless
                // when the PDO had been installed to master. 
                I8123W.I8123_InstallPDO_List(SLOT, NODE, (ushort)(0x200 + NODE), 0, 1, (byte)255, 0, 0, Entry_Data, BlockMode);
                
                // Install PDO remote ISR
                REMOTEPDOISR = new I8123W.RemotePDOISR(UserREMOTEPDOISR);
                ret = I8123W.I8123_InstallRemotePDOISR(SLOT, REMOTEPDOISR);
                if (ret != 0)
                {
                    errstr = "Install remote PDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }
                
                // The "EntryUse" parameter and "EntryData" parameter are useless
                // when the PDO had been installed to master.
                I8123W.I8123_InstallPDO_List(SLOT, NODE, (ushort)(0x180 + NODE), 1, 1, (byte)255, 0, 0, Entry_Data, BlockMode);

                timer1.Enabled = true;

            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_ShutdownMaster(SLOT);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_Clear.Enabled = false;
                Combo_Node.SelectedIndex = 0;

                List_Monitor.Items.Clear();

                timer1.Enabled = false;
            }
        }

        private void Btn_Clear_Click(object sender, EventArgs e)
        {
            List_Monitor.Items.Clear();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (ISR_Str != "")
            {
                List_Monitor.Items.Add(ISR_Str);
                ISR_Str = "";
            }
        }

    }
}