﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using I8123W_Net;

namespace SYNC_Protocol
{
    public partial class Form1 : Form
    {
        byte SLOT;
        byte BAUD;
        byte NODE;

        byte BlockMode = 1;

        byte[] IsAdd = new byte[128];

        ushort DelayTime;
        ushort ret;
        string errstr;

        public Form1()
        {
            InitializeComponent();
        }

        private void UpdataList()
        {
            ushort [] a_SyncId = new ushort[5], a_Timer = new ushort[5];
            uint [] a_Times = new uint[5];

            List_SYNCID.Items.Clear();

            ret = I8123W.I8123_GetCyclicSYNCInfo(SLOT, a_SyncId, a_Timer, a_Times, BlockMode);
            if (ret == 0)
            {
                ushort i;
                string Temp_Str;

                for (i = 0; i < 5; i++)
                {
                    if (a_SyncId[i] == 0)
                    {
                        break;
                    }

                    Temp_Str = "SYNC: " + a_SyncId[i].ToString("X3") + " , Timer: " + a_Timer[i].ToString("D") + " , Times: " + a_Times[i].ToString("D");
                    List_SYNCID.Items.Add(Temp_Str);
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ushort i;

            SLOT = 0;
            BAUD = 0;
            NODE = 1;

            Combo_Slot.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = false;

            Combo_Slot.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;

            Btn_AddNode.Enabled = false;
            Btn_AddNode.Visible = true;
            Btn_Remove.Enabled = false;

            Pal_SYNC.Enabled = false;

            for (i = 0; i < 128; i++)
            {
                IsAdd[i] = 0;
            }
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            I8123W.I8123_ShutdownMaster(SLOT);
        }

        private void Combo_Slot_SelectedIndexChanged(object sender, EventArgs e)
        {
            SLOT = (byte)(Combo_Slot.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);

            switch (BAUD)
            {
                case 0:
                    DelayTime = 15;
                    break;
                case 1:
                    DelayTime = 7;
                    break;
                case 2:
                    DelayTime = 3;
                    break;
                case 3:
                case 4:
                    DelayTime = 1;
                    break;
                case 5:
                case 6:
                case 7:
                    DelayTime = 0;
                    break;
            }
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);

            if (IsAdd[NODE] != 0)
            {
                ushort SyncId = 0;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_Remove.Enabled = true;

                Pal_SYNC.Enabled = true;

                ret = I8123W.I8123_GetSYNCID(SLOT, NODE, out SyncId, BlockMode);
                if (ret == 0)
                {
                    Text_SYNCID.Text = SyncId.ToString("X");
                    Text_SYNCID.SelectionStart = Text_SYNCID.TextLength;
                }
            }
            else
            {
                Btn_Remove.Enabled = false;
                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;

                Pal_SYNC.Enabled = false;
            }
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_InitMaster(SLOT, 0, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ushort i;

            ret = I8123W.I8123_ShutdownMaster(SLOT);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Slot.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;

                Pal_SYNC.Enabled = false;

                List_SYNCID.Items.Clear();

                for (i = 0; i < 128; i++)
                {
                    IsAdd[i] = 0;
                }
            }
        }

        private void Btn_AddNode_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_AddNode(SLOT, NODE, 1, DelayTime, 200, BlockMode);
            if (ret != 0)
            {
                errstr = "AddNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                ushort SyncId = 0;

                IsAdd[NODE] = 1;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_Remove.Enabled = true;

                Pal_SYNC.Enabled = true;

                ret = I8123W.I8123_GetSYNCID(SLOT, NODE, out SyncId, BlockMode);
                if (ret == 0)
                {
                    Text_SYNCID.Text = SyncId.ToString("X");
                    Text_SYNCID.SelectionStart = Text_SYNCID.TextLength;
                }
            }
        }

        private void Btn_Remove_Click(object sender, EventArgs e)
        {
            ret = I8123W.I8123_RemoveNode(SLOT, NODE, BlockMode);
            if (ret != 0)
            {
                errstr = "RemoveNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                IsAdd[NODE] = 0;

                Btn_Remove.Enabled = false;
                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;

                Pal_SYNC.Enabled = false;

                Text_SYNCID.Text = "";

                UpdataList();
            }
        }

        private void Check_Is_Dec(object sender, KeyPressEventArgs e)
        {
            if (char.IsDigit(e.KeyChar)) // check decimal value
            {
                e.Handled = false;
            }
            else
            {
                e.Handled = true;
            }
        }

        private void Check_Is_Hex(object sender, KeyPressEventArgs e)
        {
            if (char.IsDigit(e.KeyChar)) // check decimal value
            {
                e.Handled = false;
            }
            else
            {   // check hex value
                if ((e.KeyChar >= (Char)65) && (e.KeyChar <= (Char)70)) // check A ~ F 
                {
                    e.Handled = false;
                }
                else if ((e.KeyChar >= (Char)97) && (e.KeyChar <= (Char)102)) // check a ~ f
                {
                    e.Handled = false;
                }
                else
                {
                    e.Handled = true;
                }
            }
        }
        
        private void Btn_SendSYNC_Click(object sender, EventArgs e)
        {
            ushort SyncId, Timer;
            uint Times;

            SyncId = Convert.ToUInt16(Text_SYNCID.Text, 16);
            Timer = Convert.ToUInt16(Text_SYNCTimer.Text, 10);
            Times = Convert.ToUInt32(Text_SYNCTimes.Text, 10);

            ret = I8123W.I8123_SendSYNCMsg(SLOT, SyncId, Timer, Times, BlockMode);
            if (ret != 0)
            {
                errstr = SyncId.ToString("X") + " SendSYNCMsg Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                UpdataList();
            }
        }

        private void Btn_SYNCID_Click(object sender, EventArgs e)
        {
            ushort SyncId;

            SyncId = Convert.ToUInt16(Text_SYNCID.Text, 16);

            ret = I8123W.I8123_ChangeSYNCID(SLOT, NODE, SyncId, BlockMode);
            if (ret != 0)
            {
                errstr = "ChangeSYNCID Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }

            ret = I8123W.I8123_GetSYNCID(SLOT, NODE, out SyncId, BlockMode);
            if (ret == 0)
            {
                Text_SYNCID.Text = SyncId.ToString("X");
                Text_SYNCID.SelectionStart = Text_SYNCID.TextLength;
            }
        }
    }
}